/* agenda.cpp - programa principal de agenda telefonica */

#define TAM_BUSCA   15

#include "tela.h"
#include "arquivo.h"
#include "registro.h"
#include "memoria.h"

memoria pilha;

static void pesquisa (int &i_ind, int &i_base)

{

  char cpz_entidade [TAM_ENTIDADE + 1];

  tela pesquisar ("PESQUISA");

  char cpz_busca [TAM_BUSCA] = "";

  int i_result;

  do {

    if ((i_result = pesquisar.edita_busca (cpz_busca, TAM_BUSCA)) == SAI) {
      i_ind = i_base = 0;
      return;
    }

  } while (i_result != DESCE);

  pilha.prepara_busca ();

  do {

    int i_pos_ind, i_pos_base;

    if (pilha.procura_chave (cpz_entidade, i_pos_ind, i_pos_base, cpz_busca)
	== OK) {

      pesquisar.mostra_entidade (cpz_entidade, TAM_ENTIDADE);
      if (pesquisar.confirmacao ("Este e o registro procurado") == SIM) {
	i_ind = i_pos_ind;
	i_base = i_pos_base;
	return;
      };

    }
    else {
      i_ind = i_base = 0;
      return;
    }

  } while (VERDADEIRO);
}



void cria_chaves_pilha (void)

{

  disco d_pilha (NOME_ARQ_INDICE, NOME_ARQ_BASE);
  registro pilha_reg;

  d_pilha.inic_arq_ind ();

  char *buffer = new char [pilha_reg.tamanho_reg_ind () + 1];

  int i_pos;

  while ((i_pos = d_pilha.leia_arq_ind (buffer, pilha_reg.tamanho_reg_ind ()))
	  != ERRO) {

    pilha_reg.coloque_buffer_ind (buffer);

    char *entidade = pilha_reg.traga_nome_entidade ();

    int pos_base = pilha_reg.traga_posicao_base ();

    pilha.insere_chave (entidade, i_pos, pos_base);

    delete entidade;

  }

  delete buffer;

}




void incluir (void)

{

  disco saida (NOME_ARQ_INDICE, NOME_ARQ_BASE);

  while (VERDADE) {

    tela incluir ("INCLUSAO");
    registro incluir_reg;

    char entidade [TAM_ENTIDADE + 1] = "",
	 telefone [TAM_TELEFONE + 1] = "",
	 linha    [4] [TAM_LINHA + 1] = {"", "", "", ""};

    int i_posicao = 0;
    int i_retorno;

    do {

      switch (i_posicao) {

	case (0): i_retorno = incluir.edita_entidade (entidade, TAM_ENTIDADE);
		  break;
	case (1): i_retorno = incluir.edita_telefone (telefone, TAM_TELEFONE);
		  break;
	case (2): i_retorno = incluir.edita_linha (0, linha [0], TAM_LINHA);
		  break;
	case (3): i_retorno = incluir.edita_linha (1, linha [1], TAM_LINHA);
		  break;
	case (4): i_retorno = incluir.edita_linha (2, linha [2], TAM_LINHA);
		  break;
	case (5): i_retorno = incluir.edita_linha (3, linha [3], TAM_LINHA);
		  break;

      }

      switch (i_retorno) {

	case (SOBE):

	  if (i_posicao > 0)
	    i_posicao--;

	  break;

	case (DESCE):

	  i_posicao++;

	  break;

	case (SAI):

	  return;

      }

    } while (i_posicao < 6);


    if (incluir.confirmacao ("Confirma a inclusao") == NAO)
      return;

    incluir_reg.coloque_nome_entidade (entidade);
    incluir_reg.coloque_telefone (telefone);
    for (int i = 0; i < 4; i++)
      incluir_reg.coloque_linha (i, linha [i]);


    char *buffer_base = incluir_reg.traga_buffer_base ();

    int i_posicao_grav = saida.grave_arq_base (pilha.num_registros (),
					   buffer_base,
					   incluir_reg.tamanho_reg_base());

    incluir_reg.coloque_posicao_base (i_posicao_grav);

    delete buffer_base;

    char *buffer_ind = incluir_reg.traga_buffer_ind ();

    int pos = saida.grave_arq_ind (pilha.num_registros (), buffer_ind,
				   incluir_reg.tamanho_reg_ind ());

    delete buffer_ind;

    pilha.insere_chave (entidade, pos, i_posicao_grav);

  };

}

void excluir (void)
{

  disco saida (NOME_ARQ_INDICE, NOME_ARQ_BASE);

  while (VERDADE) {

    int i_pos_ind, i_pos_base;

    pesquisa (i_pos_ind, i_pos_base);

    if (i_pos_ind == 0 || i_pos_base == 0)
      return;

    tela excluir ("EXCLUIR");
    registro excluir_reg;

    char *entidade, *telefone, *linha [4];

    char *buffer_ind = new char [excluir_reg.tamanho_reg_ind () + 1];
    char *buffer_base = new char [excluir_reg.tamanho_reg_base () + 1];

    saida.leia_arq_base (buffer_base, excluir_reg.tamanho_reg_base (),
			 i_pos_base);
    excluir_reg.coloque_buffer_base (buffer_base);

    saida.leia_arq_ind (buffer_ind, excluir_reg.tamanho_reg_ind (),
			  i_pos_ind);
    excluir_reg.coloque_buffer_ind (buffer_ind);

    entidade = excluir_reg.traga_nome_entidade ();
    excluir.mostra_entidade (entidade, TAM_ENTIDADE);
    telefone = excluir_reg.traga_telefone ();
    excluir.mostra_telefone (telefone, TAM_TELEFONE);
    for (int i = 0; i < 4; i ++) {
      linha [i] = excluir_reg.traga_linha (i);
      excluir.mostra_linha (i, linha [i], TAM_LINHA);
    }

    delete entidade;
    delete telefone;
    for (int j = 0; j < 4; j++)
      delete linha [j];
    delete buffer_ind;
    delete buffer_base;

    if (excluir.confirmacao ("Confirma a exclusao do registro") == NAO)
      return;


    pilha.retira_chave (i_pos_ind, i_pos_base);

    saida.delete_reg_arq_ind (excluir_reg.tamanho_reg_ind (), i_pos_ind);
    saida.delete_reg_arq_base (excluir_reg.tamanho_reg_base (), i_pos_base);

  };
}


void consultar (void)

{

  disco entrada (NOME_ARQ_INDICE, NOME_ARQ_BASE);

  while (VERDADE) {

    int i_pos_ind, i_pos_base;

    pesquisa (i_pos_ind, i_pos_base);

    if (i_pos_ind == 0 || i_pos_base == 0)
      return;

    tela consultar ("CONSULTAR");
    registro consultar_reg;

    char *entidade, *telefone, *linha [4];

    char *buffer_ind = new char [consultar_reg.tamanho_reg_ind () + 1];
    char *buffer_base = new char [consultar_reg.tamanho_reg_base () + 1];

    entrada.leia_arq_base (buffer_base, consultar_reg.tamanho_reg_base (),
			 i_pos_base);
    consultar_reg.coloque_buffer_base (buffer_base);

    entrada.leia_arq_ind (buffer_ind, consultar_reg.tamanho_reg_ind (),
			  i_pos_ind);
    consultar_reg.coloque_buffer_ind (buffer_ind);

    entidade = consultar_reg.traga_nome_entidade ();
    consultar.mostra_entidade (entidade, TAM_ENTIDADE);
    telefone = consultar_reg.traga_telefone ();
    consultar.mostra_telefone (telefone, TAM_TELEFONE);
    for (int i = 0; i < 4; i ++) {
      linha [i] = consultar_reg.traga_linha (i);
      consultar.mostra_linha (i, linha [i], TAM_LINHA);
    }

    consultar.mensagem ("Para continuar:");

    delete entidade;
    delete telefone;
    for (int j = 0; j < 4; j++)
      delete linha [j];
    delete buffer_ind;
    delete buffer_base;

  }
}

void alterar (void)
{

  disco entr_saida (NOME_ARQ_INDICE, NOME_ARQ_BASE);

  while (VERDADE) {

    int i_pos_ind, i_pos_base;

    pesquisa (i_pos_ind, i_pos_base);

    if (i_pos_ind == 0 || i_pos_base == 0)
      return;

    tela alteracao ("ALTERARCAO");
    registro alterar_reg;

    char *entidade, *telefone, *linha [4];

    char *buffer_ind = new char [alterar_reg.tamanho_reg_ind () + 1];
    char *buffer_base = new char [alterar_reg.tamanho_reg_base () + 1];

    entr_saida.leia_arq_base (buffer_base, alterar_reg.tamanho_reg_base (),
			      i_pos_base);
    alterar_reg.coloque_buffer_base (buffer_base);

    entr_saida.leia_arq_ind (buffer_ind, alterar_reg.tamanho_reg_ind (),
			     i_pos_ind);
    alterar_reg.coloque_buffer_ind (buffer_ind);

    delete buffer_ind;
    delete buffer_base;

    entidade = alterar_reg.traga_nome_entidade ();
    telefone = alterar_reg.traga_telefone ();
    for (int i = 0; i < 4; i ++)
      linha [i] = alterar_reg.traga_linha (i);

    int i_posicao = 0;
    int i_retorno;

    do {

      switch (i_posicao) {

	case (0): i_retorno = alteracao.edita_entidade (entidade,
							TAM_ENTIDADE);
		  break;
	case (1): i_retorno = alteracao.edita_telefone (telefone,
							TAM_TELEFONE);
		  break;
	case (2): i_retorno = alteracao.edita_linha (0, linha [0],
						     TAM_LINHA);
		  break;
	case (3): i_retorno = alteracao.edita_linha (1, linha [1],
						     TAM_LINHA);
		  break;
	case (4): i_retorno = alteracao.edita_linha (2, linha [2],
						     TAM_LINHA);
		  break;
	case (5): i_retorno = alteracao.edita_linha (3, linha [3],
						     TAM_LINHA);
		  break;

      }

      switch (i_retorno) {

	case (SOBE):

	  if (i_posicao > 0)
	    i_posicao--;

	  break;

	case (DESCE):

	  i_posicao++;

	  break;

	case (SAI):

	  delete entidade;
	  delete telefone;
	  for (int m = 0; m < 4; m++)
	    delete linha [m];

	  return;

      }

    } while (i_posicao < 6);


    if (alteracao.confirmacao ("Confirma a alteracao") == NAO) {
      delete entidade;
      delete telefone;
      for (int l = 0; l < 4; l++)
	delete linha [l];
      return;
    }

    alterar_reg.coloque_nome_entidade (entidade);
    alterar_reg.coloque_telefone (telefone);
    for (int j = 0; j < 4; j++)
      alterar_reg.coloque_linha (j, linha [j]);


    char *rec_buffer_base = alterar_reg.traga_buffer_base ();

    entr_saida.grave_arq_base (rec_buffer_base,
			       i_pos_base,
			       alterar_reg.tamanho_reg_base());

    alterar_reg.coloque_posicao_base (i_pos_base);

    delete rec_buffer_base;

    char *rec_buffer_ind = alterar_reg.traga_buffer_ind ();

    entr_saida.grave_arq_ind (rec_buffer_ind,
			      i_pos_ind,
			      alterar_reg.tamanho_reg_ind ());

    delete rec_buffer_ind;

    pilha.retira_chave (i_pos_ind, i_pos_base);

    pilha.insere_chave (entidade, i_pos_ind, i_pos_base);

    delete entidade;
    delete telefone;
    for (int k = 0; k < 4; k++)
      delete linha [k];
  }
}

void listar (void)
{
  tela listagem ("LISTAGEM");
  listagem.mensagem ("Para colocar os nomes em ordem alfabetica:");
  pilha.ordene ();
  if (listagem.confirmacao ("Confirma a listagem") == SIM)
    pilha.liste_memoria ();
}

void main (void)

{

  cria_chaves_pilha ();

  int i_comando;

  do {

    tela principal ("PRINCIPAL");

    i_comando = principal.menu ();

    switch (i_comando) {
      case (INCLUIR): incluir (); break;
      case (ALTERAR): alterar (); break;
      case (CONSULTAR): consultar (); break;
      case (EXCLUIR): excluir (); break;
      case (LISTAR): listar (); break;
    }

  } while (i_comando != FIM);

}